// Copyright (c) 2007 Google Inc.
// All rights reserved
// 
// This file is part of the Google Desktop SDK and may be freely copied and used.
// To download the latest version of the SDK please visit 
// http://desktop.google.com/developer.html

var DEFAULT_DOWNLOAD_INTERVAL_MS = 60000;

function PhotoSlideshow(downloadIntervalMs) {
  this.displayArea = null;
  this.photos = [];
  this.photoSwitchInvervalMs = 10000;
  this.onDisplayPhotoHandler = null;
  this.currentPhotoIndex = -1;
  this.dlNextPhotoId = -1;
  this.showNextPhotoId = -1;
  
  if (downloadIntervalMs === undefined)
    downloadIntervalMs = DEFAULT_DOWNLOAD_INTERVAL_MS;
  
  this.startPhotoDownload(downloadIntervalMs);
}

PhotoSlideshow.prototype.displayingPhoto = function() {
  return (this.displayArea != null && this.currentPhotoIndex != -1);
}

// photos must have already been queued prior to downloading via the
// addPhoto or addPhotoToFront methods
PhotoSlideshow.prototype.downloadPhoto = function(imageUrl) {
  for (var i = 0; i < this.photos.length; ++i) {
    if (this.photos[i].imageUrl == imageUrl) {
      return this.downloadPhotoByIndex(i);
    }
  }
  
  return false;
} 

PhotoSlideshow.prototype.downloadPhotoByIndex = function(index) {
  // never re-download a photo.. thus, this slideshow won't work well if the URL you are 
  // fetching from is changing and you want to reflect that. On the flip side, we get a 
  // slideshow where the client doesn't need to care if a photo has or has not been 
  // downloaded -- they can demand that it's downloaded and ready for display even if it
  // already is
  var self = this;

  if (this.photos[index].image != null)
    return true;
    
  var selectedItem = this.photos[index];

  var photoDownloader = new SimpleHTTPRequest();
  debug.trace("Downloading photo " + selectedItem.imageUrl);
  photoDownloader.request(selectedItem.imageUrl, onDownloadedPhoto, true, selectedItem.imageUrl);
  
  function onDownloadedPhoto(data, tag) {
    var id = -1;
    for (var i = 0; i <= self.photos.length - 1; i++) {
      if (self.photos[i].imageUrl == tag) {
        id = i;
        break;
      }
    }

    if (id == -1)
      throw 0;
    debug.trace("Got photo data " + tag);
    self.photos[id].image = data;
    
    if (self.photos[id].showImmediatelyAfterDownload) {
      self.photos[id].showImmediatelyAfterDownload = false;
      self.showPhotoByIndex(id);
      // reset the slideshow if it's running, so that we don't do a super-fast switch 
      if (self.showNextPhotoId !== -1)
        self.startSlideshow(self.photoSwitchInvervalMs, self.onDisplayPhotoHandler);
    }
  }
  
  return true;  
}

PhotoSlideshow.prototype.startPhotoDownload = function(intervalMs) {
  var self = this;
  
  // photo downloading interval
  this.stopPhotoDownload();
  this.dlNextPhotoId = setInterval(downloadNextPhoto, intervalMs);
  
  function downloadNextPhoto() {
    // Scan through all entries to find the next non-downloaded photo
    var selectedIndex = -1;
    for (var i = 0; ((i <= self.photos.length - 1) && (selectedIndex == -1)); i++) {
      if ((self.photos[i].image == null) && (self.photos[i].imageUrl != null))
        selectedIndex = i;
    }
    
    // No images are queued to be downloaded
    if (selectedIndex == -1) {
      debug.trace("No images not downloaded");
      return;
    }
    
    self.downloadPhotoByIndex(selectedIndex);
  }
}

PhotoSlideshow.prototype.setDisplayArea = function(displayArea) {
  this.displayArea = displayArea;
}

PhotoSlideshow.prototype.createPhotoObject = function(uri, extraData) {
  var obj = new Object();

  debug.trace("added photo " + uri);

  obj.imageUrl = uri;      // The URL to the image that gets downloaded
  obj.image = null;         // The image data which was downloaded
  obj.showImmediatelyAfterDownload = false;
  obj.extraData = extraData;
  return obj;
}

PhotoSlideshow.prototype.addPhoto = function(uri, extraData) {
  var obj = this.createPhotoObject(uri, extraData);
  this.photos.push(obj);
  return true;
}

PhotoSlideshow.prototype.getCurrentPhotoExtraData = function() {
  if (this.currentPhotoIndex <= this.photos.length - 1) {
    if (this.photos[this.currentPhotoIndex]) {
      return this.photos[this.currentPhotoIndex].extraData;
    }
  }

  return null;
}

// will get downloaded ASAP
PhotoSlideshow.prototype.addPhotoToFront = function(uri, extraData) {
  var obj = this.createPhotoObject(uri, extraData);
  this.photos.splice(0, 0, obj);
  return true;
}

PhotoSlideshow.prototype.removePhoto = function(uri) {
  for (var i = 0; i < this.photos.length; ++i) {
    if (this.photos[i].imageUrl == uri) {
      this.photos.splice(i, 1);
      
      // handle all the edge cases that happen when you remove a photo, like dealing with the fact
      // that maybe there are no more photos, and stop showing the photo that is removed
      var currentPhotoIndex = this.currentPhotoIndex;
      if (this.photos.length == 0)
        this.currentPhotoIndex = -1; // reset the display
      if (i == currentPhotoIndex) {
        this.displayArea.src = null;
        this.showPhoto(1);
      }
      return true;
    }
  }
  return false;
}

PhotoSlideshow.prototype.showPhoto = function(uri) {
 for (var i = 0; i < this.photos.length; ++i) {
    if (this.photos[i].imageUrl == uri) {
      if (this.photos[i].image == null) {
        this.photos[i].showImmediatelyAfterDownload = true;
        this.downloadPhotoByIndex(i);
      } else {
        this.showPhotoByIndex(i);
      }
      break;
    }
  }
}

PhotoSlideshow.prototype.showNextPhoto = function(increment) {
  if (this.photos.length == 0)
    return;
  
  var nextIndex = -1;
  var self = this;
  function nextPhotoIterator(currentPhoto) {
    // when nothing's being displayed, there's a negative index.. so we just start at 0
    var next = 0;
    if (currentPhoto >= 0) {
      next = (currentPhoto + increment) % self.photos.length;
      // JScript does negative modulos incorrectly - handle that by just going 
      // to the end, which is fine for our purposes
      if (currentPhoto + increment < 0)
        next = self.photos.length - 1;
    }
    
    return next;
  }

  // See if there are any images remaining queued up
  var processed = 0;
  for (var i = nextPhotoIterator(this.currentPhotoIndex); ( (i != this.currentPhotoIndex) && (processed < this.photos.length) && (nextIndex == -1)); i = nextPhotoIterator(i)) {
    if (this.photos[i].image)
      nextIndex = i;
    processed++;
  }

  if (nextIndex == -1) {
    // No images ready at all
    // debug.trace("No images ready");
    return;
  } else {
    return this.showPhotoByIndex(nextIndex);
  }
}

PhotoSlideshow.prototype.showPhotoByIndex = function(index) {
  this.currentPhotoIndex = index;

  debug.trace("showing photo " + index);
  this.displayArea.src = this.photos[this.currentPhotoIndex].image;

  // call the client's handler with the photo info    
  this.onDisplayPhotoHandler(this.photos[this.currentPhotoIndex].imageUrl, this.photos[this.currentPhotoIndex].extraData);
}

PhotoSlideshow.prototype.startSlideshow = function(intervalMs, onDisplayPhotoHandler) {
  var self = this;
  
  this.photoSwitchInvervalMs = intervalMs;
  this.stopSlideshow();
  this.onDisplayPhotoHandler = onDisplayPhotoHandler;
  
  this.showNextPhotoId = setInterval(showNextPhotoCallback, intervalMs);

  function showNextPhotoCallback() {
    self.showNextPhoto(1);
  }
}

PhotoSlideshow.prototype.stopSlideshow = function() {
  clearInterval(this.showNextPhotoId); 
  this.showNextPhotoId = -1;
}

PhotoSlideshow.prototype.stopPhotoDownload = function() {
  clearInterval(this.dlNextPhotoId);
  this.dlNextPhotoId = -1;
}

PhotoSlideshow.prototype.isShowingPhoto = function() {
  return this.currentPhotoIndex != -1;
}

PhotoSlideshow.prototype.getPhotoCount = function() {
  return this.photos.length;
}
